/*
 * AUTHOR: Sergey Erokhin                 esl@pisem.net,pk8020@gmail.com
 * &Korvet Team                                              2000...2005
 * ETALON Korvet Emulator                         http://pk8020.narod.ru
 * ---------------------------------------------------------------------
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 *
 */
#include <stdio.h>
#include <stdlib.h>


#include "korvet.h"

#ifdef DBG
 #include "dbg/dbg.h"
#endif

//#include "i8080.h"


#ifdef DBG
extern word dbg_HERE;  //     
extern int  dbg_TRACE;  //    
extern int  dbg_Cause;
extern int  dbg_CauseAddr;
#endif

//#define ROM_FILE_NAME   "data/kontur.rom"
//#define ROM_FILE_NAME   "data/korvet2.rom"
//#define ROM_FILE_NAME   "data/korvet.rom"
char RomFileName[1024]="data/rom.rom";
char MapperFileName[1024]="data/mapper.mem";
#define noDEBUG_MEMORY

#define ROMSIZE         (24*1024)

//    (     )

#define M_RAM            0
#define M_ROM0           1      //  ROM0 0000-1fff
#define M_ROM1           2      //  ROM1 2000-3fff
#define M_ROM2           3      //  ROM2 4000-5fff
#define M_KEYBOARD       4
#define M_PORTBASE       5      //   
#define M_REGBASE        6      //  
#define M_ACZU           7
#define M_GZU            8


extern int DBG_Trace ;
extern int TraceCause;
extern int CauseAddr;
extern int InDBG;

extern screen,font;
#ifdef DEBUG_MEMORY_LOW
// EXTERNAL !!!!!!!!!!!!!!!!!!!!!!!
int NCREG;
// EXTERNAL !!!!!!!!!!!!!!!!!!!!!!!

//  
void GZU_Write(int Addr,byte Value){};
void ACZU_Write(int Addr,byte Value){};
void LUT_Write(byte Value){};

byte GZU_read(int Addr){};
byte ACZU_read(int Addr){};
#else
extern unsigned int NCREG;
#endif

int     SYSREG;                 //    
                                //   (0-31)
                                //      (>>2)&0x1f

byte    MemMapper[32][256];     //     

byte    RAM[RAMSIZE];           // RAM 
byte    ROM[ROMSIZE];           // ROM 

byte    BreakPoint[RAMSIZE];    // DBG::  

// ================================================================================= RAM part

// --------------------------------------------------------------------------------- SYSREG Part
//     SYSREGa
int Mapper_Init(void) {
 int i;
 FILE *MFILE;
 if ((MFILE=fopen(MapperFileName,"rb")) == NULL) return ERROR;
 for (i=0;i<32;i++) {
   if (fread(MemMapper[i],1,256,MFILE) != 256) return ERROR;
 }
 fclose(MFILE);
 SYSREG=0;
 return OK;
}
// ================================================================================= SYSREG part

// --------------------------------------------------------------------------------- ROM part
//     
int ROM_Init(char *RomFileName) {
 int i;
 FILE *MFILE;
 if ((MFILE=fopen(RomFileName,"rb")) == NULL) return ERROR;
 fread(ROM,1,ROMSIZE,MFILE);
 fclose(MFILE);
 return OK;
}
// ================================================================================= ROM part

// --------------------------------------------------------------------------------- RAM part
//  
int RAM_Init(void) {
 int i;
 for (i=0;i<RAMSIZE;i++) RAM[i]=0;
 return OK;
}
// ================================================================================= RAM part

// --------------------------------------------------------------------------------- Emulator_Write
//     
void Emulator_Write(int Addres,byte Value)
{
  int RamType;
#ifdef DBG
  if (!InDBG && (BreakPoint[Addres] & bpWRMEM) )  {dbg_TRACE=1;dbg_Cause=3;dbg_CauseAddr=Addres;} // SetTrace
#endif


  //     80% 
  if ( 0 == (RamType=MemMapper[SYSREG][(Addres&0xff00)>>8])) {
    RAM[Addres&0xffff]=Value;
  } else {
  switch (RamType) {
    case  M_RAM     :
    case  M_ROM0    :               //     ROM   RAM
    case  M_ROM1    :               //     ROM   RAM
    case  M_ROM2    :               //     ROM   RAM
    case  M_KEYBOARD:               //     KEY   RAM
                      {RAM[Addres&0xffff]=Value;break;}
    case  M_REGBASE : {
           //       
           //      0
           if ((Addres & (1<<2)) == 0) { LUT_Write(Value);
#ifdef DEBUG_MEMORY
  textprintf(screen,font,500,10,15,"LUT: = %02x",Value);
#endif
           };       /*A2, xxFB*/
           if ((Addres & (1<<6)) == 0) { NCREG =Value;
#ifdef DEBUG_MEMORY
  textprintf(screen,font,500,20,15,"NCREG: = %02x",NCREG);
#endif
           };           /*A6, xxBF*/
           if ((Addres & (1<<7)) == 0) { SYSREG=(Value>>2)&0x1f;
#ifdef DEBUG_MEMORY
  textprintf(screen,font,500,30,15,"SYSREG: = %02x",SYSREG<<2);
#endif
           }; /*A7, xx7F*/
           break;
         }
    case M_PORTBASE : {
           //        
           //   3,A4,A5 - 8 
           switch (Addres & 0x38) {
             case 0<<3:	Timer_Write(Addres,Value);break; //xx00..03
             case 1<<3:	PPI3_Write (Addres,Value);break; //xx08..0B
             case 2<<3:	RS232_Write(Addres,Value);break; //xx10..11
             case 3<<3:	FDC_Write  (Addres,Value);break; //xx18..1B
             case 4<<3:	LAN_Write  (Addres,Value);break; //xx20..21
             case 5<<3:	PIC_Write  (Addres,Value);break; //xx28..29
             case 6<<3:	PPI2_Write (Addres,Value);break; //xx30..33
             case 7<<3:	PPI1_Write (Addres,Value);break; //xx39..3B
           }
           break;
         }
    case  M_GZU     : {GZU_Write (Addres,Value);break;}
    case  M_ACZU    : {ACZU_Write(Addres,Value);break;}
  }
  }
}

// ================================================================================= Emulator_Write

// --------------------------------------------------------------------------------- Emulator_Read
//     
byte Emulator_Read(int Addres)
{
  byte Value;
  int RamType;

#ifdef DBG
  if (!InDBG && (BreakPoint[Addres] & bpRDMEM) ) {dbg_TRACE=1;dbg_Cause=2;dbg_CauseAddr=Addres;} // SetTrace
#endif
  //    - 85% 
  if ( 0 == (RamType=MemMapper[SYSREG][(Addres&0xff00)>>8])) {
    Value=RAM[Addres&0xffff];
  } else {
  switch (RamType) {
    case  M_RAM     : {Value=RAM[Addres&0xffff];break;}
    case  M_ROM0    :               
    case  M_ROM1    :               
    case  M_ROM2    : {Value=ROM[Addres&0xffff];break;}              
    case  M_KEYBOARD: {Value=KEYBOARD_Read(Addres);break;}
    case  M_REGBASE : {
           //          ???
           //   .
          Value=RAM[Addres&0xffff];
          break;
         }
    case M_PORTBASE : {
           //        
           //   3,A4,A5 - 8 
           switch (Addres & 0x38) {
             case 0<<3:	Value=Timer_Read(Addres);break;	//xx00..03
             case 1<<3:	Value=PPI3_Read (Addres);break; //xx08..0B
             case 2<<3:	Value=RS232_Read(Addres);break; //xx10..11
             case 3<<3:	Value=FDC_Read  (Addres);break; //xx18..1B
             case 4<<3:	Value=LAN_Read  (Addres);break; //xx20..21
             case 5<<3:	Value=PIC_Read  (Addres);break; //xx28..29
             case 6<<3:	Value=PPI2_Read (Addres);break; //xx30..33
             case 7<<3:	Value=PPI1_Read (Addres);break; //xx38..3B
           }
           break;
         }
    case  M_GZU     : {Value=GZU_Read(Addres);break;}
    case  M_ACZU    : {Value=ACZU_Read(Addres);break;}
  }
  }
  return Value;
}
// ================================================================================= Emulator_Read


void Memory_Init(void)
{
 Mapper_Init();
 RAM_Init();
 ROM_Init(RomFileName);
}

byte RD_BreakPoint(int Addr)           { return BreakPoint[Addr&0xffff];};
byte WR_BreakPoint(int Addr,byte Value){ BreakPoint[Addr&0xffff]=Value;};


#ifdef DEBUG_MEMORY_LOW
// MAIN & test routine ----------------------------------------------- MAIN
int main(int argc, char **argv)
{
 Mapper_Init();
 RAM_Init();
 ROM_Init(ROM_FILE_NAME);
}
#endif
